/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.bean;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import net.smartlab.web.Enumeration;
import net.smartlab.web.StringEnumeration;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 */
public class ConverterManager {

	/**
	 * @link aggregation <{net.smartlab.web.bean.Converter}>
	 * @directed directed
	 * @supplierCardinality 0..*
	 */
	private Map converters = new HashMap();

	/**
	 * TODO documentation
	 * 
	 * @param type
	 * @param value
	 * @param locale
	 * @return
	 * @throws ConversionException
	 */
	public Object convert(Class type, Object value, Locale locale) throws ConversionException {
		if (value != null) {
			try {
				if (type.isArray()) {
					return this.getConverter(type).convert(type, value, locale);
				} else {
					if (type.equals(String.class)) {
						return this.getConverter(value.getClass()).convert(type, value, locale);
					} else {
						return this.getConverter(type).convert(type, value, locale);
					}
				}
			} catch (NullPointerException npe) {
				throw new ConversionException("unavailable converter for " + type);
			}
		} else {
			return null;
		}
	}

	private Converter getConverter(Class type) throws ConversionException {
		Converter converter = (Converter)converters.get(type);
		if (converter == null) {
			if (type.equals(Object.class)) {
				throw new ConversionException();
			} else {
				return getConverter(type.getSuperclass());
			}
		} else {
			return converter;
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @return
	 */
	static public ConverterManager getDefault() {
		ConverterManager manager = new ConverterManager();

		// Booleans
		manager.converters.put(java.lang.Boolean.TYPE, new BooleanConverter(false));
		manager.converters.put(java.lang.Boolean.class, new BooleanConverter(false));
		manager.converters.put(boolean[].class, new BooleanArrayConverter());

		// Chars
		manager.converters.put(java.lang.Character.TYPE, new CharConverter());
		manager.converters.put(java.lang.Character.class, new CharConverter());
		manager.converters.put(char[].class, new CharArrayConverter());

		// Bytes
		manager.converters.put(java.lang.Byte.TYPE, new ByteConverter((byte)0));
		manager.converters.put(java.lang.Byte.class, new ByteConverter((byte)0));
		manager.converters.put(byte[].class, new ByteArrayConverter());

		// Shorts
		manager.converters.put(java.lang.Short.TYPE, new ShortConverter((short)0));
		manager.converters.put(java.lang.Short.class, new ShortConverter((short)0));
		manager.converters.put(short[].class, new ShortArrayConverter());

		// Integers
		manager.converters.put(java.lang.Integer.TYPE, new IntConverter(0));
		manager.converters.put(java.lang.Integer.class, new IntConverter(0));
		manager.converters.put(int[].class, new IntArrayConverter());

		// Longs
		manager.converters.put(java.lang.Long.TYPE, new LongConverter(0));
		manager.converters.put(java.lang.Long.class, new LongConverter(0));
		manager.converters.put(long[].class, new LongArrayConverter());

		// Floats
		manager.converters.put(java.lang.Float.TYPE, new FloatConverter(0));
		manager.converters.put(java.lang.Float.class, new FloatConverter(0));
		manager.converters.put(float[].class, new FloatArrayConverter());

		// Doubles
		manager.converters.put(java.lang.Double.TYPE, new DoubleConverter(0));
		manager.converters.put(java.lang.Double.class, new DoubleConverter(0));
		manager.converters.put(double[].class, new DoubleArrayConverter());

		// Strings
		manager.converters.put(java.lang.String.class, new StringConverter());
		manager.converters.put(String[].class, new StringArrayConverter());

		// Dates and times
		manager.converters.put(java.util.Date.class, new DateConverter());
		manager.converters.put(java.sql.Date.class, new DateConverter());
		manager.converters.put(java.sql.Time.class, new TimeConverter());
		manager.converters.put(java.sql.Timestamp.class, new DateTimeConverter());

		// Enumerations
		manager.converters.put(Enumeration.class, new EnumerationConverter());
		manager.converters.put(StringEnumeration.class, new StringEnumerationConverter());

		// Currencies
		manager.converters.put(BigDecimal.class, new BigDecimalConverter());
		manager.converters.put(BigDecimal[].class, new BigDecimalArrayConverter());
		return manager;
	}
}